/********************************************************************
 Example program to illustrate how an application can change the
 run-time and stored Flash values for Ethernet IP, mask, Gateway
 and DNS. 
 
 THIS EXAMPLE APPLIES TO TOOLS RELEASE 1.95 OR LATER
 
 INTRODUCTION FOR CONFIGURATION RECORDS AND INTERFACE BLOCKS
 - The NetBurner device contains runtime and stored (Flash) system
   configuration parameters. Interface Blocks are used at runtime,
   and Configuration Records are stored in Flash. At boot time,
   data from the Configuration Records are copied to Interface
   Blocks.

 - There are 3 Configuration Records as defined in netinterfaces.h:
     #define CONFIG_IF_ID_ETHERNET  (0)
     #define CONFIG_IF_ID_WIFI      (1)
     #define CONFIG_IF_ID_ETHERNET2 (2)
     
   The order of these configuration records is fixed, regardless of
   the Interface Block number. For example, if you have only a WiFi
   interface, you still use Configuration Record 1. 
   
 - Configuration Records are numbered 0, 1 and 2. Interface Blocks
   are numbered 1, 2, 3. There is no correlation between this 
   numbering. Interface Block 3 could reference Configuration Record
   1, depending on the order of interface registration calls by your
   application.

 - This example will provide a menu through the serial debug port
   that enables you to set/clear static IP settings, and start/stop
   the DHCP Client service. 

********************************************************************/
#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>

// Make sure to include these header files
#include <bsp.h>
#include <string.h>
#include <utils.h>
#include <system.h>
#include <netinterface.h>

extern "C" { 
   void UserMain(void * pd);
}

// Application name
const char * AppName = "Change IP Example";


// Variable to indicate if appliction tried to obtain runtime IP settings
// from a DHCP server.
bool AssignedDHCP = FALSE;


// Since we want to be able to start and stop DHCP, we need to create out
// own DHCP Object instead of using the GetDHCPAdderss() function which
// handles this automatically.
DhcpObject *pDhcpObj;

// Add a device name for DNS 
const char *DeviceName = "NetBurner";
extern const char *pDHCPOfferName;    // point this at above name in UserMain



/*-------------------------------------------------------------------
 Display runtime IP values
 This function demonstrates two methods to read the runtime IP 
 values: 
 1. Using the read-only Interface function calls
 2. Using the GetInterfaceBlock() function to get a pointer to an 
    Interface Record.
 -------------------------------------------------------------------*/
void DisplayRuntimeIPSettings(int InterfaceNumber)
{
   iprintf("\r\n\r\n--- RUNTIME IP SETTINGS ---\r\n");
   if ( AssignedDHCP )
      iprintf("Values assigned by DHCP Server\r\n");

   // Display current runtime values using Interface read-only functions
   iprintf("IP runtime settings using Interface Functions for interface %d:\r\n", 
            InterfaceNumber);
   iprintf("IP:   "); ShowIP(InterfaceIP(InterfaceNumber)); iprintf("\r\n");
   iprintf("Mask: "); ShowIP(InterfaceMASK(InterfaceNumber)); iprintf("\r\n");
   iprintf("Gate: "); ShowIP(InterfaceGate(InterfaceNumber)); iprintf("\r\n");
   iprintf("DNS:  "); ShowIP(InterfaceDNS(InterfaceNumber)); iprintf("\r\n");

   // Display current runtime values by getting a pointer to the Interface Block
   // and accessing it's variables.
   iprintf("\r\nIP runtime settings using GetInterfaceBlock() for interface %d:\r\n", 
           InterfaceNumber);
   InterfaceBlock *ib = GetInterFaceBlock(InterfaceNumber);
   iprintf("IP:   "); ShowIP(ib->netIP); iprintf("\r\n");
   iprintf("Mask: "); ShowIP(ib->netIpMask); iprintf("\r\n");
   iprintf("Gate: "); ShowIP(ib->netIpGate); iprintf("\r\n");
   iprintf("DNS:  "); ShowIP(ib->netDNS); iprintf("\r\n");
   iprintf("Interface Name: %s\r\n", ib->InterfaceName);
   iprintf("\r\n");
}


/*-------------------------------------------------------------------
 Display Flash IP values
 This function demonstrates two methods to read the IP settings 
 stored in the Flash Configuration Record: 
 1. Using the gConfigRec global read-only structure
 2. Using the RawGetConfig() functoin to obtain a pointer to a
    specific Configuration Record.
 -------------------------------------------------------------------*/
void DisplayFlashIPSettings(int RecordNumber)
{
   // Note that gConfigRec only applies to ConfigRecord 0
   iprintf("\r\n--- FLASH IP SETTINGS ---\r\n");
   iprintf("These values will be 0 if you are using DHCP\r\n");
   iprintf("IP Flash settings using gConfigRec for Record 0:\r\n");
   iprintf("IP:   "); ShowIP(gConfigRec.ip_Addr); iprintf("\r\n");
   iprintf("Mask: "); ShowIP(gConfigRec.ip_Mask); iprintf("\r\n");
   iprintf("Gate: "); ShowIP(gConfigRec.ip_GateWay); iprintf("\r\n");
   iprintf("DNS:  "); ShowIP(gConfigRec.ip_DNS_server); iprintf("\r\n");

   // Note that ConfigRecord structures start at 0 for the first interface
   ConfigRecord *cr = RawGetConfig(RecordNumber);
   iprintf("\r\nIP Flash settings using RawGetConfig() for Record %d:\r\n", 
           RecordNumber);
   iprintf("IP:   "); ShowIP(cr->ip_Addr); iprintf("\r\n");
   iprintf("Mask: "); ShowIP(cr->ip_Mask); iprintf("\r\n");
   iprintf("Gate: "); ShowIP(cr->ip_GateWay); iprintf("\r\n");
   iprintf("DNS:  "); ShowIP(cr->ip_DNS_server); iprintf("\r\n");
}



/*-------------------------------------------------------------------
 Change the runtime IP settings. 
 This function will display the current IP address and mask, 
 change the runtime variables to new values, then display the
 new values. 
 -------------------------------------------------------------------*/
void ChangeRuntimeIPSettings(int InterfaceNumber, IPADDR IpAddr, IPADDR IpMask, IPADDR IpGate, IPADDR IpDNS)
{
   iprintf("\r\nChanging IP runtime settings for interface %d:\r\n", 
           InterfaceNumber);

   // Display current values
   InterfaceBlock *ib = GetInterFaceBlock(InterfaceNumber);
   iprintf("Old Settings:\r\n");  
   iprintf("   IP:   "); ShowIP(ib->netIP); iprintf("\r\n");
   iprintf("   Mask: "); ShowIP(ib->netIpMask); iprintf("\r\n");
   iprintf("   Gway: "); ShowIP(ib->netIpGate); iprintf("\r\n");
   iprintf("   DNS:  "); ShowIP(ib->netDNS); iprintf("\r\n");
   iprintf("   Interface Name: %s\r\n", ib->InterfaceName);
   
   // Change to new values 
   ib->netIP = IpAddr;
   ib->netIpMask = IpMask;
   ib->netIpGate = IpGate;
   ib->netDNS = IpDNS;

   // Display new values. At this point, you can communicate with the
   // device using the new ip address and mask.
   iprintf("New Settings:\r\n");  
   iprintf("   IP:   "); ShowIP(ib->netIP); iprintf("\r\n");
   iprintf("   Mask: "); ShowIP(ib->netIpMask); iprintf("\r\n");
   iprintf("   Gway: "); ShowIP(ib->netIpGate); iprintf("\r\n");
   iprintf("   DNS:  "); ShowIP(ib->netDNS); iprintf("\r\n");
   iprintf("   Interface Name: %s\r\n", ib->InterfaceName);
}


/*-------------------------------------------------------------------
 Store new IP address and mask settings to the configuration record 
 in Flash. In most cases you would have already changed the runtime 
 values to the same settings.
 -------------------------------------------------------------------*/
void ChangeFlashIPSettings(int RecordNumber, IPADDR IpAddr, IPADDR IpMask, IPADDR IpGate, IPADDR IpDNS)
{
   iprintf("\r\nChanging Flash settings using RawGetConfig() for Record %d:\r\n", 
           RecordNumber);

   // Get pointer to Configuration Record
   ConfigRecord *cr = RawGetConfig(RecordNumber);

   // Display current Flash values
   iprintf("Old Settings:\r\n");  
   iprintf("   IP:   "); ShowIP(cr->ip_Addr); iprintf("\r\n");
   iprintf("   Mask: "); ShowIP(cr->ip_Mask); iprintf("\r\n");
   iprintf("   Gate: "); ShowIP(cr->ip_GateWay); iprintf("\r\n");
   iprintf("   DNS : "); ShowIP(cr->ip_DNS_server); iprintf("\r\n");
   
  // create new config record and copy data
   ConfigRecord NewRec;  
   memcpy( &NewRec, cr, sizeof( NewRec ) ); 

   // Change parameters
   NewRec.ip_Addr = IpAddr;
   NewRec.ip_Mask = IpMask;
   NewRec.ip_GateWay = IpGate;
   NewRec.ip_DNS_server = IpDNS;

   // Write new values to Flash system configuration sector
   UpdateConfigRecord_Num(&NewRec, RecordNumber);

   // Display current Flash values
   iprintf("New Settings:\r\n");  
   iprintf("   IP:   "); ShowIP(cr->ip_Addr); iprintf("\r\n");
   iprintf("   Mask: "); ShowIP(cr->ip_Mask); iprintf("\r\n");
   iprintf("   Gate: "); ShowIP(cr->ip_GateWay); iprintf("\r\n");
   iprintf("   DNS : "); ShowIP(cr->ip_DNS_server); iprintf("\r\n");
  
   // You do not need to reboot if you change both the runtime 
   // and flash values. This function call requires #include <bsp.h>
   //   ForceReboot();
}


/*-------------------------------------------------------------------
 DisplayUserMenu
 Displays menu through the serial port to interact with program.
 -------------------------------------------------------------------*/
void DisplayUserMenu()
{
   iprintf("\r\n--- Main Menu ---\r\n");
   iprintf("1. Display Runtime Settings\r\n");
   iprintf("2. Display Flash Settings\r\n");
   iprintf("3. Change Runtime Static Settings\r\n");
   iprintf("4. Change Flash Static Settings\r\n");
   iprintf("5. Change Flash & Runtime Static Settings to 0.0.0.0\r\n");
   iprintf("6. Start DHCP Client Servce\r\n");
   iprintf("   and attempt to get a DHCP address\r\n");
   iprintf("7. Stop DHCP Client Service\r\n");
}


/*-------------------------------------------------------------------
 MyStartDHCP
 Most applications can just use the GetDHCPAddress() function at boot
 time to enable the DHCP Client. If you require more control over the 
 DHCP service, such as starting and stopping, then a few lines of 
 code are required to create a DhcpObject and check a semaphore to
 determine if the DHCP request was successful. This is essentially
 what the GetDHCPAddress() function does.
 -------------------------------------------------------------------*/
void MyStartDHCP( InterfaceBlock *ib )
{
   int FirstInterface = GetFirstInterface();  // Get first interface identifier

   // The following lines of code are essentially what the GetDHCPAddress() function
   // does to make the DHCP process easier.
   pDhcpObj = new  DhcpObject( FirstInterface ); 
   pDhcpObj->StartDHCP();    // Start DHCP

   // Pend on semaphore to verify an address was obtained
   if ( OSSemPend( &( pDhcpObj->NotifySem ), 10 * TICKS_PER_SECOND ) == OS_TIMEOUT ) //Wait 10 seconds
   {
      iprintf("\r\n\r\n*** WARNING ***\r\n");
      iprintf("IP Address was set to 0.0.0.0, and a DHCP server could not be found.\r\n");
      iprintf("Device does not have a valid IP address.\r\n\r\n");
   }
   else
   {
      iprintf("DHCP assigned the following values:\r\n");
      iprintf("IP:   "); ShowIP(ib->netIP); iprintf("\r\n");
      iprintf("Mask: "); ShowIP(ib->netIpMask); iprintf("\r\n");
      iprintf("Gate: "); ShowIP(ib->netIpGate); iprintf("\r\n");
      iprintf("DNS:  "); ShowIP(ib->netDNS); iprintf("\r\n");
      iprintf("Interface Name: %s\r\n", ib->InterfaceName);
      iprintf("\r\n");
      AssignedDHCP = TRUE;
  }
}


/*-------------------------------------------------------------------
 UserMain task
 -------------------------------------------------------------------*/
void UserMain(void * pd)
{
   // This will initalize with flash parameters for IP, Mask, DNS and Gateway
   InitializeStack();  

   pDHCPOfferName = DeviceName;   // Host name for DNS

   int FirstInterface = GetFirstInterface();  // Get first interface identifier
   InterfaceBlock *ib = GetInterFaceBlock(FirstInterface);  // Get interface data
   
   if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
   {
      iprintf("\r\nNo static IP address set, attempting DHCP\r\n");
      MyStartDHCP( ib );
   }

   OSChangePrio(MAIN_PRIO);
   EnableAutoUpdate();
   StartHTTP();

   DisplayUserMenu();
   while (1)
   {
       char c = getchar();
       switch (c)
       {
          case '1': // Display runtime IP values
             DisplayRuntimeIPSettings(FirstInterface);
             break;
                  
          case '2': // Display Flash IP values
             // This example uses only 1 Configuration Record, 0
             DisplayFlashIPSettings(0);
             break;
                    
          case '3':    // Change the runtime IP address and mask
             // After this function, the device will respond to the new IP settings
             ChangeRuntimeIPSettings(FirstInterface,              // Interface Block
                                     AsciiToIp("10.1.1.24"),      // New IP address
                                     AsciiToIp("255.255.255.0"),  // New IP mask
                                     AsciiToIp("10.1.1.1"),       // New IP gateway
                                     AsciiToIp("10.1.1.2"));      // New IP dns
             break;
                    
          case '4':    // Change the Flash IP address and mask
             // If the device reboots, the new values will be in effect
             ChangeFlashIPSettings(0,               // Use first Config Record, 0
                                   AsciiToIp("10.1.1.24"),      // New IP address
                                   AsciiToIp("255.255.255.0"),  // New IP mask
                                   AsciiToIp("10.1.1.1"),       // New IP gateway
                                   AsciiToIp("10.1.1.2"));      // New IP dns
             break;
             
          case '5':    // Set all flash values to 0.0.0.0
             ChangeFlashIPSettings(0,               // Use first Config Record, 0
                                   AsciiToIp("0.0.0.0"),      // New IP address
                                   AsciiToIp("0.0.0.0"),      // New IP mask
                                   AsciiToIp("0.0.0.0"),      // New IP gateway
                                   AsciiToIp("0.0.0.0"));     // New IP dns

             ChangeRuntimeIPSettings(FirstInterface,              // Interface Block
                                     AsciiToIp("0.0.0.0"),      // New IP address
                                     AsciiToIp("0.0.0.0"),  // New IP mask
                                     AsciiToIp("0.0.0.0"),       // New IP gateway
                                     AsciiToIp("0.0.0.0"));      // New IP dns
             break;
             
         case '6':  // Start DHCP Client service
             if ( !AssignedDHCP )
             {
                iprintf("Contacting DHCP Server ...\r\n");
                MyStartDHCP( ib );
             }
             else
             {
                iprintf("\r\n*** Error: DHCP Client service is alredy running\r\n");
             }
             break;

         case '7':  // Stop DHCP Client service
             if ( AssignedDHCP )
             {
                pDhcpObj->StopDHCP();    // Stop DHCP
                AssignedDHCP = FALSE;    // Flag for this application
                iprintf("\r\nDHCP Release sent and DHCP Client service has been stopped\r\n");
                DisplayRuntimeIPSettings(FirstInterface);
             }
             else
             {
                iprintf("\r\n*** Error: DHCP Client service is not running\r\n");
             }
             break;
                    
          default:  DisplayUserMenu();
       }      
   }      
}








